#include <stdio.h>
#include <linux/i2c-dev.h> 
#include <fcntl.h>
#include <math.h>   //-lm
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <wiringPiI2C.h>
#include "direction.h"


/*****************************************************************
*   
*****************************************************************/
int open_compass() {
int	compass = wiringPiI2CSetup(0x1e);  //pZT[ZbgAbv(I2CVXe)
	if ((wiringPiI2CReadReg8(compass, 0x09) % 2 )== 1 ){
		printf("COMPASS opened\n");
		return 0;
	}else{
		printf("failed to open COMPASS\n");
		return -1;
	}
}


int compass_init(void)
{
    char *i2cfile = "/dev/i2c-1";
    unsigned char i2caddr = 0x1e;
    unsigned char data;
    
    if ((m_fd = open(i2cfile, O_RDWR)) < 0) {
        printf("Faild to open i2c port\n");
        return -1;
    }
    if (ioctl(m_fd, I2C_SLAVE, i2caddr) < 0) {
        printf("Unable to get bus access to talk to slave\n");
        return -1;
    }
    data = compass_read(0x0a);
    if (data != 0x48) {
        printf("Identification Register 0x0a check failure [%02x]", data);
        return -1;
    }
    data = compass_read(0x0b);
    if (data != 0x34) {
        printf("Identification Register 0x0b check failure [%02x]", data);
        return -1;
    }
    data = compass_read(0x0c);
    if (data != 0x33) {
        printf("Identification Register 0x0c check failure [%02x]", data);
        return -1;
    }
    compass_write(0x00, 0xe0); /* set default val to Config Reg A */
    compass_write(0x02, 0x00); /* set continuous mode to Mode Reg */

    m_nDirection = 0;

    return 0;
}
/*****************************************************************
*   pl 0..359 0:k
*****************************************************************/
int compass_get(void)
{
    int nCorrectionValue = 20;
    unsigned char status;
    int data[3];

    status = compass_read(0x09);
    if (status | 0x1) {
        compass_readData(data);
        m_nDirection = (int)(atan2((double)data[0] + nCorrectionValue, ((double)data[2] + nCorrectionValue)*(-1)) * 180 / 3.14159265358979 + 180);
    }

    return m_nDirection;
}
/*****************************************************************
*   write
*****************************************************************/
static int compass_write(unsigned char address, unsigned char data)
{
    unsigned char buf[2];
    buf[0] = address;
    buf[1] = data;
    if ((write(m_fd, buf, 2)) != 2) {
        printf("Error writing to i2c slave\n");
        return -1;
    }

    return 0;
}
/*****************************************************************
*   read
*****************************************************************/
static unsigned char compass_read(unsigned char address)
{
    unsigned char buf[1];
    buf[0] = address;
    if ((write(m_fd, buf, 1)) != 1) {
        printf("Error writing to i2c slave\n");
    }
    if (read(m_fd, buf, 1) != 1) {
        printf("Error reading from i2c slave\n");
    }

    return buf[0];
}
/*****************************************************************
*   data read
*****************************************************************/
static void compass_readData(int *idata)
{
    unsigned char i, addr, data[6];

    for (i = 0; i<6; i++) {
        addr = 0x03 + i;
        data[i] = compass_read(addr);
    }
    idata[0] = ((int)data[0] << 24 | (int)data[1] << 16) >> 16;
    idata[1] = ((int)data[2] << 24 | (int)data[3] << 16) >> 16;
    idata[2] = ((int)data[4] << 24 | (int)data[5] << 16) >> 16;
}

///////////////
int compass_angle(int compass_reset){
    if(compass_get() >= compass_reset){
   	    return (compass_get()-compass_reset);
   	}else{
       return (360+compass_get()-compass_reset);  
    }
}
