#include <stdio.h>
#include <unistd.h>
#include <wiringPiI2C.h>
#include "uss.h"

#define WIDTH 100

// ̎ԊԊu[ms]
static const int t_uss = 50;

// ͈͂̉E[cm]
static const int uss_min = 16;
static const int uss_max = 600;

// ZTMIRSS܂ł̋[cm]
static const int dist_center = 10;

static int fd_l, fd_r, fd_f, fd_b;

int uss_open_l() {
    fd_l = wiringPiI2CSetup(0x71);

    // \tgEFArW̊mF
    if (wiringPiI2CReadReg8(fd_l, 0x00) == 0x06) {
        printf("USS_L opened\n");
        return 0;
    } else {
        printf("failed to open USS_L\n");
        return -1;
    }
}

int uss_open_r() {
    fd_r = wiringPiI2CSetup(0x70);

    // \tgEFArW̊mF
    if (wiringPiI2CReadReg8(fd_r, 0x00) == 0x06) {
        printf("USS_R opened\n");
        return 0;
    } else {
        printf("failed to open USS_R\n");
        return -1;
    }
}

int uss_open_b() {
    fd_b = wiringPiI2CSetup(0x72);

    // \tgEFArW̊mF
    if (wiringPiI2CReadReg8(fd_b, 0x00) == 0x06) {
        printf("USS_B opened\n");
        return 0;
    } else {
        printf("failed to open USS_B\n");
        return -1;
    }
}

int uss_open_f() {
    fd_f = wiringPiI2CSetup(0x73);

    // \tgEFArW̊mF
    if (wiringPiI2CReadReg8(fd_f, 0x00) == 0x06) {
        printf("USS_F opened\n");
        return 0;
    } else {
        printf("failed to open USS_F\n");
        return -1;
    }
}

long uss_get_l() {
    unsigned char lowbyte, highbyte;
    long val;

    wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
    usleep(t_uss * 1000);

    lowbyte = wiringPiI2CReadReg8(fd_l, 0x03);
    highbyte = wiringPiI2CReadReg8(fd_l, 0x02);
    val = lowbyte + highbyte * 256;

    if (val < uss_min || val > uss_max) {
        return -1;
    } else {
        return val + dist_center;
    }
}

long uss_get_r() {
    unsigned char lowbyte, highbyte;
    long val;

    wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
    usleep(t_uss * 1000);

    lowbyte = wiringPiI2CReadReg8(fd_r, 0x03);
    highbyte = wiringPiI2CReadReg8(fd_r, 0x02);
    val = lowbyte + highbyte * 256;

    if (val < uss_min || val > uss_max) {
        return -1;
    } else {
        return val + dist_center;
    }
}

long uss_get_b() {
    unsigned char lowbyte, highbyte;
    long val;

    wiringPiI2CWriteReg8(fd_b, 0x00, 0x51);
    usleep(t_uss * 1000);

    lowbyte = wiringPiI2CReadReg8(fd_b, 0x03);
    highbyte = wiringPiI2CReadReg8(fd_b, 0x02);
    val = lowbyte + highbyte * 256;

    if (val < uss_min || val > uss_max) {
        return -1;
    } else {
        return val + dist_center;
    }
}

long uss_get_f() {
    unsigned char lowbyte, highbyte;
    long val;

    wiringPiI2CWriteReg8(fd_f, 0x00, 0x51);
    usleep(t_uss * 1000);

    lowbyte = wiringPiI2CReadReg8(fd_f, 0x03);
    highbyte = wiringPiI2CReadReg8(fd_f, 0x02);
    val = lowbyte + highbyte * 256;

    if (val < uss_min || val > uss_max) {
        return -1;
    } else {
        return val + dist_center;
    }
}

int uss_get_all(long *uss_l, long *uss_r, long *uss_b, long *uss_f) {
    unsigned char lowbyte_l, lowbyte_r, lowbyte_b, lowbyte_f, highbyte_l, highbyte_r, highbyte_b, highbyte_f;
    long val_l, val_r, val_b, val_f;

    wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
    wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
    wiringPiI2CWriteReg8(fd_b, 0x00, 0x51);
    wiringPiI2CWriteReg8(fd_f, 0x00, 0x51);
    usleep(t_uss * 1000);

    lowbyte_l = wiringPiI2CReadReg8(fd_l, 0x03);
    highbyte_l = wiringPiI2CReadReg8(fd_l, 0x02);
    val_l = lowbyte_l + highbyte_l * 256;
    lowbyte_r = wiringPiI2CReadReg8(fd_r, 0x03);
    highbyte_r = wiringPiI2CReadReg8(fd_r, 0x02);
    val_r = lowbyte_r + highbyte_r * 256;
    lowbyte_b = wiringPiI2CReadReg8(fd_b, 0x03);
    highbyte_b = wiringPiI2CReadReg8(fd_b, 0x02);
    val_b = lowbyte_b + highbyte_b * 256;
    lowbyte_f = wiringPiI2CReadReg8(fd_f, 0x03);
    highbyte_f = wiringPiI2CReadReg8(fd_f, 0x02);
    val_f = lowbyte_f + highbyte_f * 256;

    if (val_l < uss_min || val_l > uss_max) {
        *uss_l = -1;
    } else {
        *uss_l = val_l + dist_center;
    }
    if (val_r < uss_min || val_r > uss_max) {
        *uss_r = -1;
    } else {
        *uss_r = val_r + dist_center;
    }
    if (val_b < uss_min || val_b > uss_max) {
        *uss_b = -1;
    } else {
        *uss_b = val_b + dist_center;
    }
    if (val_f < uss_min || val_f > uss_max) {
        *uss_f = -1;
    } else {
        *uss_f = val_f + dist_center;
    }
    return 0;
}

int uss_get_wallstate() {
    long dist_r, dist_l, dist_f, dist_b;
    uss_get_all(&dist_l, &dist_r, &dist_b, &dist_f);
    if ((dist_l == -1) || (dist_r == -1)) {
        return -1;
    } else if (dist_r + dist_l <= WIDTH) {
        printf("wall=4\n");
        return 4;
    } else {
        if ((dist_r >= WIDTH)&&(dist_l >= WIDTH)) {
            printf("wall=1\n");
            return 1;
        } else if (dist_r >= WIDTH) {
            printf("wall=3\n");
            return 3;
        } else if (dist_l >= WIDTH) {
            printf("wall=2\n");
            return 2;
        } else {
            return -1;
        }
    }
}
